import dotenv from 'dotenv';
import publicRouter from './routes/public';
import privateRouter from './routes/private';
import mongoose from 'mongoose';
import express, { Response } from 'express';
import { expressjwt, Request, UnauthorizedError } from 'express-jwt';
import cors from 'cors';

dotenv.config();

try {
    await mongoose.connect(process.env.MONGO_URI!);
    console.log('Connected to MongoDB at ' + process.env.MONGO_URI);
} catch (error) {
    console.error(error);
}

const app = express();
const healthRouter = express.Router();
healthRouter.route('/healthcheck').get((_req, res) => {
    res.status(200).send('Hello there!');
});

app.use(cors());
app.use(express.json());
app.use(express.urlencoded({ extended: true }));
app.use(
    '/api', 
    healthRouter, 
    publicRouter, 
    privateRouter, 
    expressjwt({ 
        secret: process.env.SECRET!, 
        algorithms: ['HS512'],
        onExpired: (_req, err: UnauthorizedError) => {
            throw err;
        }
    }).unless({ path: [...publicRouter.stack, ...healthRouter.stack] }),
    (err: Error, req: Request, res: Response) => {
        if (err.name === 'UnauthorizedError' || !req.auth) {
            return res.status(401).json({ message: "Unauthorized" });
        }
    }
);

// For development mode, the server is started by Vite, so we only need to do this for production
if (process.env.NODE_ENV === 'production') {
    app.listen(3333, () => {
        console.log(`Listening at http://localhost:3333`);
    });
}

export const viteNodeApp = app;
